 ###############################################
#                                               #
# D I S T A N C E   D E   L E V E N S H T E I N #
#       v1.0 (04/06/2009)  par MenzAgitat       #
#                                               #
#          http://www.boulets-roxx.com          #
#        IRC:  irc.epiknet.org  #boulets        #
#              irc.teepi.net    #boulets        #
#                                               #
#     Mes scripts sont tlchargeables sur      #
#            http://www.eggdrop.fr              #
#        ou  http://www.egghelp.org             #
#                                               #
 ###############################################

#
# Ce script pourvoit l'eggdrop du package Levenshtein :
#	package provide Levenshtein 1.0
#
# Description :
#		La distance de Levenshtein mesure la similarit entre deux chanes de
#		caractres. Elle est gale au nombre minimal de caractres qu'il faut
#		supprimer, insrer ou remplacer pour passer d'une chane  l'autre.
#		( http://fr.wikipedia.org/wiki/Distance_de_Levenshtein )
#
# Intrts :
#		Permet  un correcteur orthographique de faire des suggestions pour tel mot
#		mal orthographi, en proposant d'autres mots dont la distance de Levenshtein
#		par rapport au premier est faible.
#		Permet  une pseudo-IA de type chatterbot d'avoir une tolrance
#		orthographique : si tel mot compar  tel autre a une distance infrieure 
#		une tolrance donne, on peut dcider qu'il est quivalent.
#		
# Syntaxe :
#		levenshtein::distance <string 1> <string 2>
#
# Exemples :
#			levenshtein::distance "bonjour" "bougeoir"
#			-> 4
#		Il faut manipuler 4 caractres pour passer du mot "bonjour"
#		au mot "bougeoir" :
#		BONJOUR
#		BOUJOUR		-> on remplace le N par un U
#		BOUGOUR		-> on remplace le J par un G
#		BOUGEOUR	-> on insre un E
#		BOUGEOIR	-> on remplace le U par un I
#			levenshtein::distance "antiquaire" "antikaire"
#			-> 2
#		On peut conclure de cet exemple de 10 lettres qu'il est trs similaire
#		avec le 2me mot avec une distance de 2 seulement.
#		Il est important d'avoir  l'esprit qu'une distance de 2 entre deux
#		mots de 10 lettres dnote une ressemblance, tandis qu'une distance de 2
#		entre deux mots de 3 lettres signifie qu'ils sont trs diffrents comme
#		le montre l'exemple suivant :
#			levenshtein::distance "pin" "pas"
#			-> 2
#		Comme vous le voyez, une distance de 2 ne permet pas de faire beaucoup
#		de modifications dans un mot de 10 lettres mais elle reprsente un
#		changement radical pour un mot de 3 lettres.
#		Afin de prserver la pertinence des rsultats, vous prendrez soin de
#		toujours lier la tolrance  la longueur, proportionnellement.
#			levenshtein::distance "antiquaire" "dimanche"
#			-> 8
#		Sur ce dernier exemple, on peut voir qu'il y a une distance de 8 entre
#		le 1er mot et le second qui est trs diffrent.
#

#
#	Changelog
#		1.0 	- 1re version
#

#
# LICENCE:
#		This program is free software; you can redistribute it and/or modify
#		it under the terms of the GNU General Public License as published by
#		the Free Software Foundation; either version 2 of the License, or
#		(at your option) any later version.
#
#		This program is distributed in the hope that it will be useful,
#		but WITHOUT ANY WARRANTY; without even the implied warranty of
#		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#		GNU General Public License for more details.
#

if { [lindex [split $version] 1] < 1061800 } { putloglev o * "\00304\002\[Distance de Levenshtein - ERREUR\]\002\003 La version de votre eggdrop est \00304\002[lindex [split $version] 0]\002\003; Levenshtein.tcl ne fonctionnera correctement que sur les eggdrops version 1.6.18 ou suprieure." ; return }
if { $tcl_version < 8.4 } { putloglev o * "\00304\002\[Distance de Levenshtein - ERREUR\]\002\003 Levenshtein.tcl ncessite que Tcl 8.4 (ou plus) soit install pour fonctionner. Votre version actuelle de Tcl est \00304\002$tcl_version\002\003." ; return }
package require Tcl 8.4
if {[info commands levenshtein::uninstall] eq "::levenshtein::uninstall"} { levenshtein::uninstall }

namespace eval levenshtein {
	variable scriptname "Distance de Levenshtein"
  variable version "1.0.20090604"
	package provide Levenshtein 1.0
	bind evnt - prerehash levenshtein::uninstall
	bind pub - !test_levenshtein levenshtein::test_levenshtein
	proc uninstall {args} {
		putlog "Dsallocation des ressources de \002$levenshtein::scriptname...\002"
		foreach binding [lsearch -inline -all -regexp [binds *[set ns [string range [namespace current] 2 end]]*] " (::)?$ns"] {
			unbind [lindex $binding 0] [lindex $binding 1] [lindex $binding 2] [lindex $binding 4]
		}
		package forget Levenshtein
		namespace delete ::levenshtein
	}
}

proc levenshtein::distance {string1 string2} {
	set length1 [string length $string1]
	set length2 [string length $string2]
	if { !$length1 } { return $length2 }
	if { !$length2 } { return $length1 }
	# On cre une matrice de ($length + 1) par ($length2 + 1)
	set x $length1
	while { $x > 0 } {
		set matrix_[set x]x0 $x
		incr x -1
	}
	set y $length2
	while { $y >= 0 } {
		set matrix_0x[set y] $y
		incr y -1
	}
	while { $x < $length1 } {
		incr x
		set y 1
		while { $y <= $length2 } {
			if { [string index $string1 [expr $x - 1]] eq [string index $string2 [expr $y - 1]] } { set cost 0 } { set cost 1 }
			set tmp [expr [min [subst \$matrix_[expr $x - 1]x[set y]] [subst \$matrix_[set x]x[expr $y - 1]]] + 1]
			set matrix_[set x]x[set y] [min $tmp [expr [subst \$matrix_[expr $x -1]x[expr $y - 1]] + $cost]]
			incr y
		}
	}
	return [subst \$matrix_[set length1]x[set length2]]
}

proc levenshtein::min { value1 value2 } {
	return [expr $value1 < $value2 ? $value1 : $value2]
}

proc levenshtein::test_levenshtein {nick host handle chan arg} {
	if { ($arg eq "") || ([llength $arg] > 2) } {
		puthelp "PRIVMSG $chan :\037Syntaxe\037 : \002!test_levenshtein\002 \00314<\003string 1\00314> <\003string 2\00314>\003"
		return
	}
	puthelp "PRIVMSG $chan :\00314[levenshtein::distance [lindex $arg 0] [lindex $arg 1]]\003"
}


putlog "\002*$levenshtein::scriptname v$levenshtein::version*\002 par MenzAgitat a t charg."
